==> dxtime_1.2.2-1.tar.gz: BUILD/src/gui/dialogs/recorddialog.cpp <==
/***************************************************************
 * Name:      recorddialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include "dxdefs.h"
#include "recorddialog.h"
#include "model/dbmodel.h"

IMPLEMENT_DYNAMIC_CLASS(RecordDialog, wxDialog)

BEGIN_EVENT_TABLE(RecordDialog, wxDialog)
    EVT_BUTTON(wxID_OK, RecordDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, RecordDialog::OnCancel)
    EVT_TIME_CHANGED(ID_RECORDSTART, RecordDialog::OnTime)
    EVT_TIME_CHANGED(ID_RECORDEND, RecordDialog::OnTime)
END_EVENT_TABLE()

RecordDialog::RecordDialog(int taskid, wxWindow *parent)
: wxDialog(parent, wxID_ANY, _("New record")), m_editing(false), m_id(-1), m_taskid(taskid)
{
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

RecordDialog::RecordDialog(wxWindow *parent, int id)
: wxDialog(parent, wxID_ANY, _("Edit record")), m_editing(true), m_id(id)
{
    m_taskid = idbmodel->getTime(id).taskid();
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void RecordDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxFlexGridSizer *gridSizer = new wxFlexGridSizer(2, 5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("Date:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    m_dateCtrl = new wxDatePickerCtrl(this, wxID_ANY, wxDateTime::Now(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    if(m_editing) m_dateCtrl->Disable();
    gridSizer->Add(m_dateCtrl, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("Start time:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    m_startCtrl = new wxTimePickerCtrl(this, ID_RECORDSTART, wxDateTime::Now());
    gridSizer->Add(m_startCtrl, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));
    
    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("End time:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    m_endCtrl = new wxTimePickerCtrl(this, ID_RECORDEND, wxDateTime::Now());
    gridSizer->Add(m_endCtrl, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void RecordDialog::fillControls()
{
    Time time = idbmodel->getTime(m_id);
    if(time.isValid())
    {
        m_dateCtrl->SetValue(time.start());
        m_startCtrl->SetValue(time.start());
        m_endCtrl->SetValue(time.end());
    }
}

void RecordDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(createDateTime(true).IsLaterThan(createDateTime(false)))
    {
        m_endCtrl->SetValue(createDateTime(true));
    }
    if(m_editing)
    {
        if(createDateTime(true).IsLaterThan(wxDateTime::Now()) || createDateTime(false).IsLaterThan(wxDateTime::Now()))
        {
            wxMessageDialog dialog(this, _("Time is in feature"), _("Warning"), wxOK|wxICON_WARNING);
            dialog.ShowModal();
        }
        else
        {
            if(!idbmodel->timeExists(createDateTime(true), createDateTime(false), m_id))
            {
                idbmodel->editTime(Time(m_id, createDateTime(true), createDateTime(false), m_taskid));
                EndModal(wxID_OK);
            }
            else
            {
                wxMessageDialog dialog(this, _("Time interval is already used"), _("Warning"), wxOK|wxICON_WARNING);
                dialog.ShowModal();
            }
        }
    }
    else
    {
        if(createDateTime(true).IsLaterThan(wxDateTime::Now()) || createDateTime(false).IsLaterThan(wxDateTime::Now()))
        {
            wxMessageDialog dialog(this, _("Time is in feature"), _("Warning"), wxOK|wxICON_WARNING);
            dialog.ShowModal();
        }
        else
        {
            if(!idbmodel->timeExists(createDateTime(true), createDateTime(false)))
            {
                idbmodel->addTime(Time(idbmodel->getLastId("time"), createDateTime(true), createDateTime(false), m_taskid));
                EndModal(wxID_OK);
            }
            else
            {
                wxMessageDialog dialog(this, _("Time interval is already used"), _("Warning"), wxOK|wxICON_WARNING);
                dialog.ShowModal();
            }
        }
    }
}

void RecordDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void RecordDialog::OnTime(wxDateEvent &event)
{
    if(event.GetId() == ID_RECORDSTART)
    {
        if(createDateTime(true).IsLaterThan(createDateTime(false)))
        {
            m_endCtrl->SetValue(createDateTime(true));
        }
    }
    else
    {
        if(createDateTime(true).IsLaterThan(createDateTime(false)))
        {
            m_startCtrl->SetValue(createDateTime(false));
        }
    }
}

wxDateTime RecordDialog::createDateTime(bool start)
{
    wxDateTime date = m_dateCtrl->GetValue();
    date.SetHour(start?m_startCtrl->GetValue().GetHour():m_endCtrl->GetValue().GetHour());
    date.SetMinute(start?m_startCtrl->GetValue().GetMinute():m_endCtrl->GetValue().GetMinute());
    date.SetSecond(start?m_startCtrl->GetValue().GetSecond():m_endCtrl->GetValue().GetSecond());
    return date;
}

==> dxtime_1.2.4-1.tar.gz: BUILD/src/gui/dialogs/recorddialog.cpp <==
/***************************************************************
 * Name:      recorddialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include "dxdefs.h"
#include "recorddialog.h"
#include "model/dbmodel.h"

IMPLEMENT_DYNAMIC_CLASS(RecordDialog, wxDialog)

BEGIN_EVENT_TABLE(RecordDialog, wxDialog)
    EVT_BUTTON(wxID_OK, RecordDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, RecordDialog::OnCancel)
    EVT_COMMAND(wxID_ANY, TIME_UPDATED, RecordDialog::OnTime)
END_EVENT_TABLE()

RecordDialog::RecordDialog(int taskid, wxWindow *parent)
: wxDialog(parent, wxID_ANY, _("New record")), m_editing(false), m_id(-1), m_taskid(taskid)
{
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

RecordDialog::RecordDialog(wxWindow *parent, int id)
: wxDialog(parent, wxID_ANY, _("Edit record")), m_editing(true), m_id(id)
{
    m_taskid = idbmodel->getTime(id).taskid();
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void RecordDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxFlexGridSizer *gridSizer = new wxFlexGridSizer(2, 5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("Date:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    m_dateCtrl = new wxDatePickerCtrl(this, wxID_ANY, wxDateTime::Now(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    if(m_editing) m_dateCtrl->Disable();
    gridSizer->Add(m_dateCtrl, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("Start time:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    m_startCtrl = new dxTimePickerCtrl(this, ID_RECORDSTART, wxDateTime::Now());
    gridSizer->Add(m_startCtrl, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("End time:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    m_endCtrl = new dxTimePickerCtrl(this, ID_RECORDEND, wxDateTime::Now());
    gridSizer->Add(m_endCtrl, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void RecordDialog::fillControls()
{
    Time time = idbmodel->getTime(m_id);
    if(time.isValid())
    {
        m_dateCtrl->SetValue(time.start());
        m_startCtrl->setTime(time.start());
        m_endCtrl->setTime(time.end());
    }
}

void RecordDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(createDateTime(true).IsLaterThan(createDateTime(false)))
    {
        m_endCtrl->setTime(createDateTime(true));
    }
    if(m_editing)
    {
        if(createDateTime(true).IsLaterThan(wxDateTime::Now()) || createDateTime(false).IsLaterThan(wxDateTime::Now()))
        {
            wxMessageDialog dialog(this, _("Time is in feature"), _("Warning"), wxOK|wxICON_WARNING);
            dialog.ShowModal();
        }
        else
        {
            if(!idbmodel->timeExists(createDateTime(true), createDateTime(false), m_id))
            {
                idbmodel->editTime(Time(m_id, createDateTime(true), createDateTime(false), m_taskid));
                EndModal(wxID_OK);
            }
            else
            {
                wxMessageDialog dialog(this, _("Time interval is already used"), _("Warning"), wxOK|wxICON_WARNING);
                dialog.ShowModal();
            }
        }
    }
    else
    {
        if(createDateTime(true).IsLaterThan(wxDateTime::Now()) || createDateTime(false).IsLaterThan(wxDateTime::Now()))
        {
            wxMessageDialog dialog(this, _("Time is in feature"), _("Warning"), wxOK|wxICON_WARNING);
            dialog.ShowModal();
        }
        else
        {
            if(!idbmodel->timeExists(createDateTime(true), createDateTime(false)))
            {
                idbmodel->addTime(Time(idbmodel->getLastId("time"), createDateTime(true), createDateTime(false), m_taskid));
                EndModal(wxID_OK);
            }
            else
            {
                wxMessageDialog dialog(this, _("Time interval is already used"), _("Warning"), wxOK|wxICON_WARNING);
                dialog.ShowModal();
            }
        }
    }
}

void RecordDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void RecordDialog::OnTime(wxCommandEvent &event)
{
    if(event.GetId() == ID_RECORDSTART)
    {
        if(createDateTime(true).IsLaterThan(createDateTime(false)))
        {
            m_endCtrl->setTime(createDateTime(true));
        }
    }
    else
    {
        if(createDateTime(true).IsLaterThan(createDateTime(false)))
        {
            m_startCtrl->setTime(createDateTime(false));
        }
    }
}

wxDateTime RecordDialog::createDateTime(bool start)
{
    wxDateTime date = m_dateCtrl->GetValue();
    date.SetHour(start?m_startCtrl->time().GetHour():m_endCtrl->time().GetHour());
    date.SetMinute(start?m_startCtrl->time().GetMinute():m_endCtrl->time().GetMinute());
    date.SetSecond(start?m_startCtrl->time().GetSecond():m_endCtrl->time().GetSecond());
    return date;
}
